/*
 *   SPDX-FileCopyrightText: 2015-2016 Ivan Cukic <ivan.cukic@kde.org>
 *
 *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */

#ifndef SPECIALVALUES_H
#define SPECIALVALUES_H

#include <QString>

#define GLOBAL_ACTIVITY_TAG  QStringLiteral(":global")
#define ANY_ACTIVITY_TAG     QStringLiteral(":any")
#define CURRENT_ACTIVITY_TAG QStringLiteral(":current")

#define GLOBAL_AGENT_TAG     QStringLiteral(":global")
#define ANY_AGENT_TAG        QStringLiteral(":any")
#define CURRENT_AGENT_TAG    QStringLiteral(":current")

#define ANY_TYPE_TAG         QStringLiteral(":any")


#endif // SPECIALVALUES_H

